﻿using Eco.Core.Utils;
using Eco.Gameplay.GameActions;
using Eco.Gameplay.Players;
using Eco.Gameplay.Systems.Chat;
using Eco.Gameplay.Systems.TextLinks;
using Eco.Shared.IoC;
using Eco.Shared.Localization;
using Eco.Shared.Services;
using Eco.Shared.Utils;
using Eco.Simulation.Time;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Eco.Mods.ERC.RolePlay
{
	public class RolePlayChatCommands : IChatCommandHandler
	{
		private const float defaultRange = 15f;

		[ChatCommand("Shout message for distance 50m", "s")]
		public static void Shout(User user, string message = "")
		{
			if (string.IsNullOrEmpty(message)) { return; }
			SendChatDistance(user, Localizer.Do($"{user.Name} Shout: {message}"), 50f);
		}

		[ChatCommand("Whisper message for distance 5m", "w")]
		public static void Whisper(User user, string message = "")
		{
			if (string.IsNullOrEmpty(message)) { return; }
			SendChatDistance(user, Localizer.Do($"{user.Name} Whisper: {message}"), 5f);
		}

		[ChatCommand("Chat commands for Role Play.")]
		public static void RP(User user) { }

		[ChatSubCommand("RP", "Your character action, from first person", "me")]
		public static void Me(User user, string message = "")
		{
			if (string.IsNullOrEmpty(message)) { return; }
			SendChatDistance(user, Localizer.Do($"IC: {user.Name} {message}"));
		}

		[ChatSubCommand("RP", "Action from third person view, you can describe something", "do")]
		public static void Do(User user, string message = "")
		{
			if (string.IsNullOrEmpty(message)) { return; }
			SendChatDistance(user, Localizer.Do($"IC: {message}"));
		}

		[ChatSubCommand("RP", "Attempt with two possible outcomes (Successful, Failed)", "try")]
		public static void Try(User user, string message = "")
		{
			if (string.IsNullOrEmpty(message)) { return; }
			string result = RandomUtil.CoinToss() ? "Success" : "Fail";
			SendChatDistance(user, Localizer.Do($"IC: {user.Name} try {message} and {result}"));
		}

		[ChatSubCommand("RP", "Out Of Character message", "ooc")]
		public static void Ooc(User user, string message = "")
		{
			if (string.IsNullOrEmpty(message)) { return; }
			SendChat(user, Localizer.Do($"{user.Name} OOC: {message}"));
		}

		private static Result SendChatDistance(User sender, LocString message, float range = defaultRange)
		{
			ChatMessage chatMessage = new ChatMessage() { MessageLocation = ChatMessageLocation.Normal, Text = message };
			return SendChatDistance(sender, chatMessage, range);
		}

		private static Result SendChatDistance(User sender, ChatMessage message, float range)
		{
			IEnumerable<User> onlineUsers = UserManager.OnlineUsers;
			foreach (User target in onlineUsers)
			{
				if (target == null) { continue; }
				float dst = Shared.Voxel.World.WrappedDistance(sender.Position, target.Position);
				if (dst <= range) { message.UsersWithVisibility.Add(target.Name); }
			}
			return SendChat(sender, message);
		}

		private static Result SendChat(User sender, LocString message)
		{
			ChatMessage chatMessage = new ChatMessage() { MessageLocation = ChatMessageLocation.Normal, Text = message };
			return SendChat(sender, chatMessage);
		}

		private static Result SendChat(User sender, ChatMessage message)
		{
			if (UserManager.Config.UserPermission.MuteList.Contains(sender))
			{
				LocString error = Localizer.DoStr("You are currently in the mute list and not allowed to use chat.");
				sender.Player?.Error(error);
				return Result.Fail(error);
			}
			message.Text = ProfanityFilter.Clean(message.Text);
			message.Category = MessageCategory.Chat;
			message.Tag = DefaultChatTags.General.TagName();
			message.TimeSeconds = WorldTime.Seconds;
			//message.Sender = sender.Name;
			//message.SenderUILink = sender.UILink();
			if (!string.IsNullOrEmpty(message.Text))
			{
				Result chatSentResult = new ChatSent { Tag = message.Tag, ActionLocation = sender.Position.XYZi, Citizen = sender, Message = message.Text }.TryPerform();
				if (chatSentResult)
				{
					ChatMessage processedMsg = ServiceHolder<IChatManager>.Obj.ProcessMessage(message);
					ChatManager.MessageSent.Invoke(message);
				}
				return chatSentResult;
			}
			else { return Result.FailLocStr("Empty message"); }
		}
	}
}
